@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-'''
+"""
 External Nodes Classifier
 =========================
 
@@ -25,7 +25,6 @@ does.
 
 See (admittedly degenerate and probably not complete) example:
 
-
 .. code-block:: yaml
 
     classes:
@@ -41,59 +40,77 @@ The above essentially is the same as a top.sls containing the following:
         - basepackages
         - database
 
-'''
+    base:
+      '*':
+        - basepackages
+        - database
+"""
+from __future__ import absolute_import, print_function, unicode_literals
 
-# Import python libs
+# Import Python libs
 import logging
+import shlex
 import subprocess
 
-# Import third party libs
-import yaml
+# Import Salt libs
+import salt.utils.platform
+import salt.utils.yaml
+
+if salt.utils.platform.is_windows():
+    from salt.utils.win_functions import escape_argument as _cmd_quote
+else:
+    _cmd_quote = shlex.quote
 
 log = logging.getLogger(__name__)
 
 
 def __virtual__():
-    '''
+    """
     Only run if properly configured
-    '''
-    if __opts__['master_tops'].get('ext_nodes'):
+    """
+    if __opts__["master_tops"].get("ext_nodes"):
         return True
     return False
 
 
 def top(**kwargs):
-    '''
+    """
     Run the command configured
-    '''
-    if 'id' not in kwargs['opts']:
+    """
+    if "id" not in kwargs["opts"]:
         return {}
-    cmd = '{0} {1}'.format(
-            __opts__['master_tops']['ext_nodes'],
-            kwargs['opts']['id']
+    proc = subprocess.run(
+        [
+            _cmd_quote(part)
+            for part in shlex.split(
+                __opts__["master_tops"]["ext_nodes"],
+                posix=salt.utils.platform.is_windows() is False,
             )
-    ndata = yaml.safe_load(
-            subprocess.Popen(
-                cmd,
-                shell=True,
-                stdout=subprocess.PIPE
-                ).communicate()[0])
+            + [_cmd_quote(kwargs["opts"]["id"])]
+        ],
+        stdout=subprocess.PIPE,
+        check=True,
+        shell=True,  # nosec
+    )
+    ndata = salt.utils.yaml.safe_load(proc.stdout)
     if not ndata:
-        log.info('master_tops ext_nodes call did not return any data')
+        log.info("master_tops ext_nodes call did not return any data")
     ret = {}
-    if 'environment' in ndata:
-        env = ndata['environment']
+    if "environment" in ndata:
+        env = ndata["environment"]
     else:
-        env = 'base'
+        env = "base"
 
-    if 'classes' in ndata:
-        if isinstance(ndata['classes'], dict):
-            ret[env] = list(ndata['classes'])
-        elif isinstance(ndata['classes'], list):
-            ret[env] = ndata['classes']
+    if "classes" in ndata:
+        if isinstance(ndata["classes"], dict):
+            ret[env] = list(ndata["classes"])
+        elif isinstance(ndata["classes"], list):
+            ret[env] = ndata["classes"]
         else:
             return ret
     else:
-        log.info('master_tops ext_nodes call did not have a dictionary with a "classes" key.')
+        log.info(
+            'master_tops ext_nodes call did not have a dictionary with a "classes" key.'
+        )
 
     return ret
