@@ -10,6 +10,7 @@ import re
 import shutil
 import tempfile
 
+import salt.client.ssh.client
 import salt.config
 import salt.roster
 import salt.utils.files
@@ -432,3 +433,38 @@ class SSHSingleTests(TestCase):
 
         ret = single._cmd_str()
         assert re.search('SET_PATH=""', ret)
+
+
+@skipIf(not salt.utils.path.which("ssh"), "No ssh binary found in path")
+class SSHTests(ShellCase):
+    def setUp(self):
+        self.roster = """
+            localhost:
+              host: 127.0.0.1
+              port: 2827
+            """
+        self.opts = salt.config.client_config(self.get_config_file_path("master"))
+        self.opts["selected_target_option"] = "glob"
+
+    def test_extra_filerefs(self):
+        """
+        test "extra_filerefs" are not excluded from kwargs
+        when preparing the SSH opts
+        """
+        opts = {
+            "eauth": "auto",
+            "username": "test",
+            "password": "test",
+            "client": "ssh",
+            "tgt": "localhost",
+            "fun": "test.ping",
+            "ssh_port": 22,
+            "extra_filerefs": "salt://foobar",
+        }
+        roster = os.path.join(RUNTIME_VARS.TMP_CONF_DIR, "roster")
+        client = salt.client.ssh.client.SSHClient(
+            mopts=self.opts, disable_custom_roster=True
+        )
+        with patch("salt.roster.get_roster_file", MagicMock(return_value=roster)):
+            ssh_obj = client._prep_ssh(**opts)
+            assert ssh_obj.opts.get("extra_filerefs", None) == "salt://foobar"
